@@ -2,10 +2,7 @@ package Sidef::Types::Hash::Hash {
 
     use 5.014;
 
-    use parent qw(
-      Sidef::Object::Object
-      Sidef::Convert::Convert
-      );
+    use parent qw(Sidef::Object::Object);
 
     use overload
       q{bool} => sub { scalar(CORE::keys %{$_[0]}) },
